#Import Python libraries
import requests
import pandas as pd
import numpy as np
import gspread
from oauth2client.service_account import ServiceAccountCredentials


#Enter league information
#Values for espn_s2 and swid required to get private league data.  Navigate to league on ESPN.  Right click on the page and
#select Inspect.  Click Application and find values for espn_s2 and swid in the list.
espn_s2 = "" #insert espn_s2 value in quotes
swid = "" #insert swid value in quotes
league_id = #insert value for league_id, which can be found in URL of ESPN league homepage
year = #insert value for current year


#Define URL with parameters
url = f'https://fantasy.espn.com/apis/v3/games/ffl/seasons/{year}/segments/0/leagues/{league_id}'

#Pull matchup and team data from the URL
matchup_response = requests.get(url, params={"leagueId": league_id,
                                             "seasonID" : year,
                                             "view" : "mMatchup"},
                                    cookies={"swid" : swid,
                                             "espn_s2" : espn_s2})

team_response = requests.get(url,    params={"leagueId": league_id,
                                             "seasonID" : year,
                                             "view" : "mTeam"},
                                     cookies={"swid" : swid,
                                             "espn_s2" : espn_s2},)


#Transform the responses into json
matchup_json = matchup_response.json()
team_json = team_response.json()


#Transform both of the json outputs into DataFrames
#One dataframe includes matchup data for the season
#One dataframe includes team data for the season
matchup_df = pd.json_normalize(matchup_json['schedule'])
team_df = pd.json_normalize(team_json['teams'])


#Define the column names needed from each dataframe
matchup_column_names = {
    'matchupPeriodId':'Week', 
    'away.teamId':'Team_ID', 
    'away.totalPoints':'Points_For',
    'home.teamId':'Opponent_ID', 
    'home.totalPoints':'Points_Against',
}

team_column_names = {
    'id':'id',
    'abbrev':'Name'
}


#Reindex dataframes based on column names defined above
matchup_df = matchup_df.reindex(columns=matchup_column_names).rename(columns=matchup_column_names)
team_df = team_df.reindex(columns=team_column_names).rename(columns=team_column_names)


#Add a new column for regular/playoff game based on week number
#Update as needed based on league schedule
matchup_df['Type'] = ['Regular' if week<=14 else 'Playoff' for week in matchup_df['Week']]

# Add a new column for year
matchup_df['Year'] = year

#Add a new column for team wins
matchup_df['Team_Win'] = np.where(matchup_df['Points_For']>matchup_df['Points_Against'],1,0)

#Add a new column for team losses
matchup_df['Team_Loss'] = np.where(matchup_df['Points_For']<matchup_df['Points_Against'],1,0)

#Add a new column for opponent wins
matchup_df['Opponent_Win'] = np.where(matchup_df['Points_For']<matchup_df['Points_Against'],1,0)

#Add a new column for opponent losses
matchup_df['Opponent_Loss'] = np.where(matchup_df['Points_For']>matchup_df['Points_Against'],1,0)


#Rename Team ID column to id
matchup_df = matchup_df.rename(columns={"Team_ID":"id"})


#Merge DataFrames to get team names in addition to ids
matchup_df = matchup_df.merge(team_df, on=['id'], how='left')

#Rename Name column to Team1_Name
matchup_df = matchup_df.rename(columns={'Name':'Team_Name', 'id':'Team_ID'})


#Rename Opponent ID column to id
matchup_df = matchup_df.rename(columns={"Opponent_ID":"id"})


#Merge DataFrames to get team names in addition to ids and rename Name column to Team2_Name
matchup_df = matchup_df.merge(team_df, on=['id'], how='left')
matchup_df = matchup_df.rename(columns={'Name':'Opponent_Name'})
matchup_df = matchup_df.rename(columns={'id':'Opponent_ID'})


#Add a new column to indicate Away games
matchup_df['Home_or_Away'] = 'Away'


#Create matchup_df1 that includes all teams with "Away" games for the week
matchup_df1 = matchup_df[['Year', 'Week', 'Type', 'Team_ID', 'Team_Name', 'Home_or_Away', 'Points_For',
                         'Team_Win', 'Team_Loss', 'Opponent_ID', 'Opponent_Name', 'Points_Against',
                         'Opponent_Win', 'Opponent_Loss',]]


#Create matchup_df2 that includes all teams with "Home" games for the week
matchup_df2 = matchup_df[['Year', 'Week', 'Type', 'Opponent_ID', 'Opponent_Name', 'Points_Against',
                         'Opponent_Win', 'Opponent_Loss', 'Team_ID', 'Team_Name', 'Points_For',
                         'Team_Win', 'Team_Loss']]


#Add a new column to indicate Home games
matchup_df2['Home_or_Away'] = 'Home'


#Rename columns in matchup_df2 to match matchup_df1
matchup_df2 = matchup_df2.rename(columns={'Opponent_ID':'Team_ID',
                                        'Opponent_Name':'Team_Name',
                                        'Points_Against':'Points_For',
                                        'Opponent_Win': 'Team_Win',
                                        'Opponent_Loss': 'Team_Loss',
                                        'Team_ID': 'Opponent_ID',
                                        'Team_Name':'Opponent_Name',
                                        'Points_For': 'Points_Against',
                                        'Team_Win': 'Opponent_Win',
                                        'Team_Loss': 'Opponent_Loss',})


#Reorder Columns in matchup_df2
matchup_df2 = matchup_df2[['Year', 'Week', 'Type', 'Team_ID', 'Team_Name', 'Home_or_Away', 'Points_For',
                         'Team_Win', 'Team_Loss', 'Opponent_ID', 'Opponent_Name', 'Points_Against',
                         'Opponent_Win', 'Opponent_Loss',]]


#Combine matchup_df1 and matchup_df2 dataframes so that away and home teams for each week are in one dataframe
matchup_union_dfs = pd.concat([matchup_df1, matchup_df2])


#Convert the Points_For column to a numeric data type
matchup_union_dfs["Points_For"] = pd.to_numeric(matchup_union_dfs["Points_For"])


#Calculate the median for each week and join data back to dataframe
#League dependent - the median can be used to see which teams are in the top half of points for the week
matchup_union_dfs = matchup_union_dfs.join(
    matchup_union_dfs.groupby('Week')[['Points_For']]
        .transform('median')  # Calculate the median for each week
        .rename(columns='Median'.format))  # Rename columns


#Remove duplicates from dataframe
matchup_union_dfs=matchup_union_dfs.drop_duplicates()


#Add a new column for weekly score
#League dependent
matchup_union_dfs['Score'] = np.where(matchup_union_dfs['Points_For']>matchup_union_dfs['Points_Against'],2,0)+np.where(matchup_union_dfs['Points_For']>matchup_union_dfs['Median'],1,0)


#Add weekly cumulative columns to dataframe
matchup_union_dfs['Cumulative Score'] = matchup_union_dfs['Score'].groupby(matchup_union_dfs['Team_Name']).cumsum()
matchup_union_dfs['Cumulative Wins'] = matchup_union_dfs['Team_Win'].groupby(matchup_union_dfs['Team_Name']).cumsum()
matchup_union_dfs['Cumulative Losses'] = matchup_union_dfs['Team_Loss'].groupby(matchup_union_dfs['Team_Name']).cumsum()
matchup_union_dfs['Cumulative Points For'] = matchup_union_dfs['Points_For'].groupby(matchup_union_dfs['Team_Name']).cumsum()
matchup_union_dfs['Cumulative Points Against'] = matchup_union_dfs['Points_Against'].groupby(matchup_union_dfs['Team_Name']).cumsum()


#Reindex data frame
matchup_union_dfs.reset_index(drop=True,inplace=True)


#Add a rank column based on league logic
#Each team is ranked based on their cumulative score, then cumulative wins, then points for
matchup_union_dfs['Weekly_Rank'] = (matchup_union_dfs.sort_values(['Cumulative Score','Cumulative Wins','Cumulative Points For'])
                      .groupby(['Week']).cumcount(ascending=False)+1)


#Drop dataframe columns for output
matchup_union_dfs.drop(['Opponent_Win', 'Opponent_Loss'],axis=1,inplace=True)


#Set up connection to Google Sheet
#Follow instructions in following article to generate the json keyfile name
#https://mljar.com/blog/authenticate-python-google-sheets-service-account-json-credentials/
scope = ['https://spreadsheets.google.com/feeds',
         'https://www.googleapis.com/auth/drive']
credentials = ServiceAccountCredentials.from_json_keyfile_name(
    '', scope) #enter keyfile_name in quotes
gc = gspread.authorize(credentials)
spreadsheet_key = '' #enter spreadsheet id in quotes
wks_name = '' #enter worksheet name in quotes


#Fill NaN values with blanks
matchup_union_dfs = matchup_union_dfs.fillna('')


#Put the dataframe values in a list
df_values = matchup_union_dfs.values.tolist()


#Open the Google Sheet
gs = gc.open_by_key(spreadsheet_key)


#Remove data from this year to overwrite
gs.values_clear("Export!A2670:U6256")  #Update sheet name and cells


#Append data to the Google Sheet
gs.values_append('Export', {'valueInputOption': 'RAW'}, {'values': df_values}) #Update sheet name

