#Import Python libraries
import requests
import pandas as pd
import numpy as np
import gspread
from oauth2client.service_account import ServiceAccountCredentials


#Enter league_id and year of data
league_id = #Enter league id here.  League id can be found in the URL of the ESPN Fantasy Football homepage
year = #Enter year to be exported here


#Define URL with parameters
url = f'https://fantasy.espn.com/apis/v3/games/ffl/leagueHistory/{league_id}?seasonId={year}'


#Pull team and matchup data from the URL
matchup_response = requests.get(url, params={"view": "mMatchup"})
team_response = requests.get(url, params={"view": "mTeam"})


#Transform the response into a json
matchup_json = matchup_response.json()[0]
team_json = team_response.json()[0]


#Transform both of the json outputs into DataFrames
#One dataframe includes historical matchup data
#One dataframe includes historical team data
matchup_df = pd.json_normalize(matchup_json['schedule'])
team_df = pd.json_normalize(team_json['teams'])


#Define the column names needed
matchup_column_names = {
    'matchupPeriodId':'Week', 
    'away.teamId':'Team_ID', 
    'away.totalPoints':'Points_For',
    'home.teamId':'Opponent_ID', 
    'home.totalPoints':'Points_Against',
}


team_column_names = {
    'id':'id',
    'abbrev': 'Name'
}


#Reindex based on column names defined above
matchup_df = matchup_df.reindex(columns=matchup_column_names).rename(columns=matchup_column_names)
team_df = team_df.reindex(columns=team_column_names).rename(columns=team_column_names)


#Add a new column for regular/playoff game based on week number
#Dependent on league schedule
if year<=2012 or year >=2021:
    matchup_df['Type'] = ['Regular' if week<=14 else 'Playoff' for week in matchup_df['Week']]
else:
    matchup_df['Type'] = ['Regular' if week<=13 else 'Playoff' for week in matchup_df['Week']]


#Add a new column for year
matchup_df['Year'] = year


#Add a new column for team wins
matchup_df['Team_Win'] = np.where(matchup_df['Points_For']>matchup_df['Points_Against'],1,0)


#Add a new column for team losses
matchup_df['Team_Loss'] = np.where(matchup_df['Points_For']<matchup_df['Points_Against'],1,0)


#Add a new column for opponent wins
matchup_df['Opponent_Win'] = np.where(matchup_df['Points_For']<matchup_df['Points_Against'],1,0)


#Add a new column for opponent losses
matchup_df['Opponent_Loss'] = np.where(matchup_df['Points_For']>matchup_df['Points_Against'],1,0)


#Rename Team ID column to id in preparation for merge
matchup_df = matchup_df.rename(columns={"Team_ID":"id"})


#Merge DataFrames to get team names in addition to ids and rename Name column to Team1_Name
matchup_df = matchup_df.merge(team_df, on=['id'], how='left')


#Rename columns
matchup_df = matchup_df.rename(columns={'id':'Team_ID','Opponent_ID':'id', 'Name': 'Team_Name'})


#Merge DataFrames to get team names in addition to ids and rename Name column to Team2_Name
matchup_df = matchup_df.merge(team_df, on=['id'], how='left')


#Rename Name column and change id to Opponent ID
matchup_df = matchup_df.rename(columns={'Name':'Opponent_Name', 'id':'Opponent_ID'})


#Add a new column to indicate Away games
matchup_df['Home_or_Away'] = 'Away'


#Create matchup_df1 that includes all teams with "Away" games in the league's history
matchup_df1 = matchup_df[['Year', 'Week', 'Type', 'Team_ID', 'Team_Name', 'Home_or_Away', 'Points_For',
                         'Team_Win', 'Team_Loss', 'Opponent_ID', 'Opponent_Name', 'Points_Against',
                         'Opponent_Win', 'Opponent_Loss',]]


#Create matchup_df2 that includes all teams with "Home" games in the league's history
matchup_df2 = matchup_df[['Year', 'Week', 'Type', 'Opponent_ID', 'Opponent_Name', 'Points_Against',
                         'Opponent_Win', 'Opponent_Loss', 'Team_ID', 'Team_Name', 'Points_For',
                         'Team_Win', 'Team_Loss']]


#Add a new column to indicate Home games
matchup_df2['Home_or_Away'] = 'Home'


#Rename columns in matchup_df2 to match matchup_df1
matchup_df2 = matchup_df2.rename(columns={'Opponent_ID':'Team_ID',
                                        'Opponent_Name':'Team_Name',
                                        'Points_Against':'Points_For',
                                        'Opponent_Win': 'Team_Win',
                                        'Opponent_Loss': 'Team_Loss',
                                        'Team_ID': 'Opponent_ID',
                                        'Team_Name':'Opponent_Name',
                                        'Points_For': 'Points_Against',
                                        'Team_Win': 'Opponent_Win',
                                        'Team_Loss': 'Opponent_Loss',})


#Reorder Columns
matchup_df2 = matchup_df2[['Year', 'Week', 'Type', 'Team_ID', 'Team_Name', 'Home_or_Away', 'Points_For',
                         'Team_Win', 'Team_Loss', 'Opponent_ID', 'Opponent_Name', 'Points_Against',
                         'Opponent_Win', 'Opponent_Loss',]]


#Combine matchup_df1 and matchup_df2 dataframes so that away and home teams are in one historical dataframe
matchup_union_dfs = pd.concat([matchup_df1, matchup_df2])


#Convert the Points_For column to a numeric data type
matchup_union_dfs["Points_For"] = pd.to_numeric(matchup_union_dfs["Points_For"])


#Calculate the median for each week and join data back to dataframe
#League dependent - the median can be used to see which teams are in the top half of points for the week
matchup_union_dfs = matchup_union_dfs.join(
    matchup_union_dfs.groupby('Week')[['Points_For']]
        .transform('median')  # Calculate the median for each week
        .rename(columns='Median'.format))  # Rename columns


#Remove duplicates from dataframe
matchup_union_dfs=matchup_union_dfs.drop_duplicates()


#Add a new column for weekly score
#League dependent
if year >=2016:
    matchup_union_dfs['Score'] = np.where(matchup_union_dfs['Points_For']>matchup_union_dfs['Points_Against'],2,0)+np.where(matchup_union_dfs['Points_For']>matchup_union_dfs['Median'],1,0)
else:
    matchup_union_dfs['Score'] = np.where(matchup_union_dfs['Points_For']>matchup_union_dfs['Points_Against'],1,0)


# Add weekly cumulative columns to dataframe
matchup_union_dfs['Cumulative Score'] = matchup_union_dfs['Score'].groupby(matchup_union_dfs['Team_Name']).cumsum()
matchup_union_dfs['Cumulative Wins'] = matchup_union_dfs['Team_Win'].groupby(matchup_union_dfs['Team_Name']).cumsum()
matchup_union_dfs['Cumulative Losses'] = matchup_union_dfs['Team_Loss'].groupby(matchup_union_dfs['Team_Name']).cumsum()
matchup_union_dfs['Cumulative Points For'] = matchup_union_dfs['Points_For'].groupby(matchup_union_dfs['Team_Name']).cumsum()
matchup_union_dfs['Cumulative Points Against'] = matchup_union_dfs['Points_Against'].groupby(matchup_union_dfs['Team_Name']).cumsum()


#Reindex data frame
matchup_union_dfs.reset_index(drop=True,inplace=True)


#Add a rank column based on league logic
#From season 2016 onward, each team is ranked based on their cumulative score, then cumulative wins, then points for
#Before the 2016 season, each team is ranked based on cuumulative wins and then points for
if year >=2016:
    matchup_union_dfs['Weekly_Rank'] = (matchup_union_dfs.sort_values(['Cumulative Score','Cumulative Wins','Cumulative Points For'])
                      .groupby(['Week']).cumcount(ascending=False)+1)
else:
    matchup_union_dfs['Weekly_Rank'] = (matchup_union_dfs.sort_values(['Cumulative Wins','Cumulative Points For'])
                      .groupby(['Week']).cumcount(ascending=False)+1)


#Drop dataframe columns for output
matchup_union_dfs.drop(['Opponent_Win', 'Opponent_Loss'],axis=1,inplace=True)


#Set up connection to Google Sheet
#Follow instructions in following article to generate the json keyfile name
#https://mljar.com/blog/authenticate-python-google-sheets-service-account-json-credentials/
scope = ['https://spreadsheets.google.com/feeds',
         'https://www.googleapis.com/auth/drive']
credentials = ServiceAccountCredentials.from_json_keyfile_name(
    '', scope) #enter keyfile_name in quotes
gc = gspread.authorize(credentials)
spreadsheet_key = '' #enter spreadsheet id in quotes
wks_name = '' #enter worksheet name in quotes


#Fill NaN values with blanks
matchup_union_dfs = matchup_union_dfs.fillna('')


#Put the dataframe values in a list
df_values = matchup_union_dfs.values.tolist()


#Open the Google Sheet
gs = gc.open_by_key(spreadsheet_key)


#Append data to the Google Sheet
gs.values_append('Export', {'valueInputOption': 'RAW'}, {'values': df_values})

